// 36
unit uHlavni;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, uMap, uCar, Grids,Math, ComCtrls, uPathSearcher,
  ExtCtrls, Buttons, uAbstractController, uSlimeController;

type
  TfrmMain = class(TForm)
    odlMap: TOpenDialog;
    mnuMain: TMainMenu;
    mitMapa: TMenuItem;
    mitOtevrit: TMenuItem;
    mitProgram: TMenuItem;
    mitInformace: TMenuItem;
    mitN1: TMenuItem;
    mitKonec: TMenuItem;
    drgMap: TDrawGrid;
    stbMain: TStatusBar;
    pnlTools: TPanel;
    rdgCtrlType: TRadioGroup;
    lbxMoves: TListBox;
    mitJizda: TMenuItem;
    mitZacit: TMenuItem;
    mitPrerusit: TMenuItem;
    btnStart: TButton;
    btnEnd: TButton;
    btnRideStep: TButton;
    mitKrok: TMenuItem;
    tmrAutoStep: TTimer;
    btnAutoStep: TCheckBox;
    sdlPath: TSaveDialog;
    btnSave: TButton;
    btnStepLot: TButton;
    procedure mitOtevritClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure drgMapDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure btnStartClick(Sender: TObject);
    procedure btnEndClick(Sender: TObject);
    procedure btnRideStepClick(Sender: TObject);
    procedure btnAutoStepClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnStepLotClick(Sender: TObject);
    procedure mitKonecClick(Sender: TObject);
    procedure mitInformaceClick(Sender: TObject);
	private
		fMap:TMap;
		fCar:TCar;
		fPath:TPathSearcher;
		fController:TController;
		
		procedure DoMapOpen(const fileName:string);
		procedure GuessBestCellSize;
		procedure DestroyAll;

		procedure RideStart;
		procedure RideEnd;
		procedure RideStep;
		procedure StepLot;
		procedure EnableCtrls(e:boolean);
		procedure RefreshList;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses uHelp;



{$R *.dfm}
//******************************************************************************
//******************************************************************************
procedure TfrmMain.DoMapOpen(const fileName: string);
begin
	DestroyAll;
	Screen.Cursor:=crHourGlass;
	try
		try
			fMap:=TMap.Create(fileName);
			drgMap.ColCount:=fMap.Width;
			drgMap.RowCount:=fMap.Height;
			stbMain.Panels[0].Text:=format('X: %d',[fMap.Width]);
			stbMain.Panels[1].Text:=format('Y: %d',[fMap.height]);
			fCar:=TCar.Create(fMap);
			fPath:=TPathSearcher.Create(fMap);
			fPath.Search;
			GuessBestCellSize;
		finally
			Screen.Cursor:=crDefault;
			EnableCtrls(true);
		end;
	except
		DestroyAll;
		raise;
	end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.mitOtevritClick(Sender: TObject);
begin
	if odlMap.Execute then
		DoMapOpen(odlMap.FileName);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.FormCreate(Sender: TObject);
begin
	fMap:=nil;
	EnableCtrls(true);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.FormDestroy(Sender: TObject);
begin
	DestroyAll;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.FormResize(Sender: TObject);
begin
	GuessBestCellSize;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.GuessBestCellSize;
var
	g:integer;
begin
	if fMap = nil then
		exit;
	
	g:=min(max((drgMap.Width-16) div fMap.Width,3),max((drgMap.Height-16) div fMap.Height,3));
	drgMap.DefaultColWidth:=g;
	drgMap.DefaultRowHeight:=g;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.drgMapDrawCell(Sender: TObject; ACol, ARow: Integer;
	Rect: TRect; State: TGridDrawState);

	function LookForPath(x,y:integer):boolean;
	var
		i:integer;
		ci:TCommandItem;
	begin
		result:=false;
		for i:=0 to fCar.Commands.Count-1 do begin
			ci:=TCommandItem(fCar.Commands[i]);
			if (ci.PosX=x)and(ci.PosY=y) then begin
				result:=true;
				exit;
			end;
		end;
	end;
	
var
	b:integer;
begin
	if fMap = nil then
		exit;
		
	with drgMap.Canvas do begin
		if (aCol=fCar.PosX)and(aRow=fCar.PosY) then
			Brush.Color:=clRed
		else if LookForPath(aCol,aRow) then
			Brush.Color:=$8080ff
		else if fPath.Route[aCol,aRow]<>0 then
			Brush.Color:=clYellow
		else begin
			b:=fMap.DirectMap[aCol,aRow];
			case b of
				miOut: Brush.Color:=clGreen;
				miRoad: Brush.Color:=clGray;
				miSafeTrack: Brush.Color:=clSilver;
				miFinish: Brush.Color:=clBlue;
			end;
		end;
		FillRect(rect);
	end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.DestroyAll;
begin
	FreeAndNil(fMap);
	FreeAndNil(fCar);
	FreeAndNil(fPath);
	EnableCtrls(true);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.RideEnd;
begin
	if fMap=nil then
		exit;

	FreeAndNil(fController);
		
	EnableCtrls(true);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.RideStart;
begin
	if fMap=nil then
		exit;

	EnableCtrls(false);
	fCar.ResetCar;
	RefreshList;
	drgMap.Repaint;
	
	case rdgCtrlType.ItemIndex of
		0: fController:=TSlimeController.Create(fCar,fMap,fPath);
	end;
	fController.Start;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.EnableCtrls(e: boolean);
begin
	mitZacit.Enabled:=e and (fMap<>nil);
	btnStart.Enabled:=e and (fMap<>nil);
	mitPrerusit.Enabled:=not e;
	btnEnd.Enabled:=not e;
	rdgCtrlType.Enabled:=e;
	mitKrok.Enabled:=not e;
	btnRideStep.Enabled:=not e;
	mitOtevrit.Enabled:=e;
	btnAutoStep.Enabled:=not e;
	btnAutoStep.Checked:=false;
	btnStepLot.Enabled:=not e;
	tmrAutoStep.Enabled:=btnAutoStep.Checked;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnStartClick(Sender: TObject);
begin
	RideStart;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnEndClick(Sender: TObject);
begin
	RideEnd;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnRideStepClick(Sender: TObject);
begin
	RideStep;
	drgMap.Repaint;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.RideStep;
begin
	if fMap=nil then
		exit;
	
	if fController.Step then
		btnEnd.Click;
	RefreshList;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnAutoStepClick(Sender: TObject);
begin
	tmrAutoStep.Enabled:=btnAutoStep.Checked;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.RefreshList;
var
	oi,i:integer;
	ci:TCommandItem;
begin
	oi:=lbxMoves.ItemIndex;
	lbxMoves.Items.BeginUpdate;
	lbxMoves.Items.Clear;
	for i:=0 to fCar.Commands.Count-1 do begin
		ci:=TCommandItem(fCar.Commands[i]);
		lbxMoves.Items.Add(format('%d,%d',[ci.commX,ci.commY]));
	end;
	lbxMoves.Items.EndUpdate;
	lbxMoves.ItemIndex:=Max(0,oi);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnSaveClick(Sender: TObject);
begin
	if sdlPath.Execute then
		lbxMoves.Items.SaveToFile(sdlPath.FileName);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.btnStepLotClick(Sender: TObject);
begin
	StepLot;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmMain.StepLot;
begin
	if fMap=nil then
		exit;
	screen.Cursor:=crHourGlass;
	try
		repeat
		until fController.Step;
	finally
		screen.Cursor:=crDefault;
		btnEnd.Click;
		RefreshList;
		drgMap.Repaint;
	end;
end;

procedure TfrmMain.mitKonecClick(Sender: TObject);
begin
	Close;
end;

procedure TfrmMain.mitInformaceClick(Sender: TObject);
begin
	frmHelp.ShowModal;
end;

end.
