unit uPathSearcher;

interface

uses
	Windows, Classes, SysUtils, uMap;

type
	TPathSearcherException = class(Exception);
	
	TPathSearcher = class
	private
		fMap:TMap;
		fRoute:TMapArray;
		fFound:boolean;
		fTargX,fTargY:integer;

		function PredictPathStep(lastLevel:integer):boolean;
		procedure PreparePrediction;
		procedure BackSearch;
	public
		constructor Create(xMap:TMap);
		destructor Destroy;override;

		procedure Search;
		property Route:TMapArray read fRoute;
		property Found:boolean read fFound;
		property TargetX:integer read fTargX;
		property TargetY:integer read fTargY;
	end;

implementation

//******************************************************************************
//******************************************************************************
{ TPathSearcher }
//******************************************************************************
//******************************************************************************
procedure TPathSearcher.BackSearch;
var
	y,x,w,h,level:integer;
	route:TMapArrayType;
	pathA:TMapArray;
	path:TMapArrayType;

	function Seek(dx,dy:integer):boolean;
	var
		nx,ny:integer;
	begin
		nx:=x+dx;
		ny:=y+dy;
		result:=false;
		if (nx<0)or(nx>=w)or(ny<0)or(ny>=h) then
			exit;
		if route[nx,ny]=level then begin
			result:=true;
			x:=nx;
			y:=ny;
		end;
	end;
	
begin
	w:=fRoute.Width;
	h:=fRoute.Height;
	pathA:=TMapArray.Create(w,h);
	path:=pathA.DirectMap;
	route:=fRoute.DirectMap;

	x:=fTargX;
	y:=fTargY;
	level:=route[x,y];
	repeat
		path[x,y]:=level;
		dec(level);
		if Seek(1,0) then
			continue;
		if Seek(-1,0) then
			continue;
		if Seek(0,-1) then
			continue;
		if Seek(0,1) then
			continue;
		if Seek(1,1) then
			continue;
		if Seek(1,-1) then
			continue;
		if Seek(-1,1) then
			continue;
		if Seek(-1,-1) then
			continue;
	until (x=fMap.StartX)and(y=fMap.StartY);
	path[x,y]:=1;
	
	for x:=0 to w-1 do
		for y:=0 to h-1 do
			route[x,y]:=path[x,y];

	pathA.Free;
	fRoute.DirectMap:=route;
end;

//******************************************************************************
//******************************************************************************
constructor TPathSearcher.Create(xMap: TMap);
begin
	fMap:=xMap;
	fRoute:=TMapArray.Create(fMap.Width,fMap.Height);
end;

//******************************************************************************
//******************************************************************************
destructor TPathSearcher.Destroy;
begin
	freeAndNil(fRoute);
	inherited;
end;

//******************************************************************************
//******************************************************************************
function TPathSearcher.PredictPathStep(lastLevel:integer):boolean;
var
	route:TMapArrayType;
	i,i2:integer;
	w,h:integer;
	written:boolean;

	procedure CheckAndWrite(xd,yd:integer);
	var
		x,y:integer;
	begin
		x:=i+xd;
		y:=i2+yd;
		if (x<0)or(x>=w)or(y<0)or(y>=h) then
			exit;
		if route[x,y]=-1 then begin
			fFound:=true;
			fTargX:=x;
			fTargY:=y;
			route[x,y]:=lastLevel+1;
			written:=true;
			exit;
		end else if route[x,y]=0 then begin
			route[x,y]:=lastLevel+1;
			written:=true;
		end;
	end;
	
begin
	written:=false;
	w:=fRoute.Width;
	h:=fRoute.Height;
	route:=fRoute.DirectMap;
	for i:=0 to fMap.Width-1 do
		for i2:=0 to fMap.Height-1 do
			if (route[i,i2]=lastLevel) then begin
				CheckAndWrite(1,0);
				CheckAndWrite(-1,0);
				CheckAndWrite(0,1);
				CheckAndWrite(0,-1);
				CheckAndWrite(1,1);
				CheckAndWrite(1,-1);
				CheckAndWrite(-1,-1);
				CheckAndWrite(-1,1);
			end;
	fRoute.DirectMap:=route;
	result:=written;
end;

//******************************************************************************
//******************************************************************************
procedure TPathSearcher.PreparePrediction;
var
	map:TMapArrayType;
	i,i2:integer;
begin
	fFound:=false;
	map:=fRoute.DirectMap;
	for i:=0 to fRoute.Width-1 do
		for i2:=0 to fRoute.Height-1 do
			case fMap.DirectMap[i,i2] of
				miFinish: map[i,i2]:=-1;
				miOut: map[i,i2]:=MaxInt;
			else
				map[i,i2]:=0;
			end;

	map[fMap.StartX,fMap.StartY]:=1;
	fRoute.DirectMap:=map;
end;

//******************************************************************************
//******************************************************************************
procedure TPathSearcher.Search;
var
	lastLevel:integer;
begin
	PreparePrediction;
	lastLevel:=1;
	repeat
		if not PredictPathStep(lastLevel) then
			break;
		inc(lastLevel);
	until fFound;

	if not fFound then
		raise TPathSearcherException.Create('Cl v tto map nen dosaiteln');

	BackSearch;
end;

end.
 